require('isomorphic-fetch')

const compression = require('compression')
const http = require('http')
const express = require('express')
const helmet = require('helmet')
const morgan = require('morgan')
const pkg = require('../package')
const debug = require('debug')
const serialize = require('serialize-javascript')

const appId = `${pkg.name}@${pkg.version}`

const start = config => {
  config.destroyMiddleware = config.destroyMiddleware || Promise.resolve()

  // create express application
  const app = express()

  // Helps to secure the app against XSS and other stuff
  app.use(helmet())

  // enable compression
  app.use(compression())

  // add logger
  if (!config.quiet) {
    const logFormat = ':method :url :status - :res[content-length] bytes - :response-time ms'

    // set application loggers
    config.loggers = {
      middleware: debug('server:middleware'),
      app: debug('server:app')
    }
    config.serializeState = (state, filters) => {
      // todo implement filters
      return serialize(state)
    }

    // normal logger (skipping status 400 and above)
    app.use(morgan(logFormat, {
      skip: (req, res) => req.get('prerender') || res.statusCode >= 400,
      stream: process.stdout
    }))

    // normal logger (skipping status below 400)
    app.use(morgan(logFormat, {
      skip: (req, res) => req.get('prerender') || res.statusCode < 400,
      stream: process.stderr
    }))
  }

  // set port
  app.set('port', config.port)

  // set powered by header
  app.use((req, res, next) => {
    res.setHeader('X-Powered-By', `${appId}`)
    next()
  })

  // heartbeat route
  app.get('/heartbeat', (req, res, next) => {
    res.end(`${appId} is up and running`)
  })

  // applying additional middlewares
  if (config.applyMiddleware) {
    config.applyMiddleware(app)
  }

  // apply routes
  app.use(express.static(config.webroot, {
    index: false,
    dotfiles: 'ignore'
  }))
  app.use('/', config.createMiddleware(config))

  // error handler
  app.use((err, req, res, next) => {
    res.locals.message = err.message
    res.locals.error = err
    res.status(err.status || 500)
    res.end(err.message)
  })

  // create server
  const server = http.createServer(app)
  server.listen(config.port)

  server.on('error', error => {
    if (error.code === 'EADDRINUSE') {
      console.error(config.port + ' is already in use')
      process.exit(1)
    }
    throw error
  })

  server.on('listening', () => {
    console.log('Listening on ' + config.port)
    if (process.send) {
      process.send('ready')
    }
  })

  let isShuttingDown = false
  const shutdown = () => {
    if (isShuttingDown) { return }
    isShuttingDown = true
    console.log('Server shutting down')
    config.destroyMiddleware()
      .then(() => {
        server.close(err => {
          if (err) {
            console.error(err)
            process.exit(1)
          }
          process.exit(0)
        })
      },
      err => {
        console.error(err)
        process.exit(1)
      })
  }

  process.on('SIGINT', shutdown)
  process.on('SIGTERM', shutdown)
}

module.exports = start
