#!/usr/bin/env node
const fs = require('fs-extra')
const path = require('path')
const program = require('commander')
const webpack = require('webpack')

process.on('unhandledRejection', error => {
  console.log('Unhandled Rejection:', error)
})

program
  .option('--src <path>', 'src location', 'src')
  .option('--dest <path>', 'dest location', 'server')
  .parse(process.argv)

const cwd = process.cwd()
const source = path.join(cwd, program.src)
const destination = path.join(cwd, program.dest)

// make sure the destination directory exists and is empty
fs.ensureDirSync(destination)
fs.emptyDirSync(destination)

const babelConfig = { compact: false }

// try to read .babelrc
try {
  Object.assign(
    babelConfig,
    JSON.parse(fs.readFileSync(path.join(__dirname, '..', '.babelrc'), 'utf8'))
  )
} catch (e) {
  console.log(e)
}

// try to read and apply .env from create react app
const needle = 'NODE_PATH='
let nodePath
try {
  nodePath = fs.readFileSync(path.join(cwd, '.env'), 'utf8')
    .split('\n')
    .reduce((acc, line) => {
      if (acc) { return acc }
      line = line.trim()
      if (line.indexOf(needle) === 0) {
        acc = line.replace(needle, '')
      }
      return acc
    }, null)
} catch (e) {
  // console.log(e)
}

const modules = ['node_modules']
if (nodePath) {
  modules.push(nodePath)
}

let entry
const entryFallbacks = [
  path.join(source, 'middleware/index.js'),
  path.join(source, 'server/index.js')
]
entryFallbacks.forEach(path => {
  if (entry || !fs.existsSync(path)) { return }
  entry = path
})

const compiler = webpack({
  entry,
  output: {
    path: path.join(cwd, 'server'),
    filename: 'middleware.js',
    library: 'middleware',
    libraryTarget: 'umd'
  },
  module: {
    rules: [
      {
        test: /\.jsx?$/,
        loader: 'babel-loader',
        options: babelConfig
      }
    ]
  },
  resolve: {
    modules
  },
  target: 'node'
})

console.log('Compiling server...')
compiler.run((err, stats) => {
  if (err) {
    console.error(err.stack || err)
    if (err.details) {
      console.error(err.details)
    }
    process.exit(1)
  }

  if (stats.hasErrors()) {
    console.error(stats.toString({ colors: true }))
    process.exit(1)
  }

  if (stats.hasWarnings()) {
    console.warn(stats.toJson().warnings)
  }

  console.log('done\n', stats.toString({ colors: true }))
  process.exit(0)
})
